@@ -20,45 +20,52 @@
  * 
  */
 /**
-    File Name:          7.5-2.js
-    ECMA Section:       7.5 Identifiers
-    Description:        Identifiers are of unlimited length
-                        - can contain letters, a decimal digit, _, or $
-                        - the first character cannot be a decimal digit
-                        - identifiers are case sensitive
+    File Name:          7.4.3-2-n.js
+    ECMA Section:       7.4.3
+
+    Description:
+    The following words are used as keywords in proposed extensions and are
+    therefore reserved to allow for the possibility of future adoption of
+    those extensions.
+
+    FutureReservedWord :: one of
+    case    debugger    export      super
+    catch   default     extends     switch
+    class   do          finally     throw
+    const   enum        import      try
 
     Author:             christine@netscape.com
-    Date:               11 september 1997
+    Date:               12 november 1997
 */
-    var SECTION = "7.5-3-n";
+    var SECTION = "7.4.3-2-n";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "Identifiers";
+    var TITLE   = "Future Reserved Words";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
     var testcases = getTestCases();
     test();
 
+
+
 function getTestCases() {
     var array = new Array();
     var item = 0;
-
-    array[item++] = new TestCase( SECTION,    "var 0abc",   "error",    "var 1abc" );
+    array[item++] = new TestCase( SECTION,  "var debugger = true",     "error",    "var debugger = true" );
     return ( array );
 }
 
 function test() {
     for ( tc=0; tc < testcases.length; tc++ ) {
         testcases[tc].actual = eval( testcases[tc].actual );
-
         testcases[tc].passed = writeTestCaseResult(
                             testcases[tc].expect,
                             testcases[tc].actual,
-                            testcases[tc].description +":  "+
+                            testcases[tc].description +" = "+
                             testcases[tc].actual );
 
-        testcases[tc].reason += ( testcases[tc].passed ) ? "" : " ignored chars after line terminator of single-line comment";
+        testcases[tc].reason += ( testcases[tc].passed ) ? "" : "wrong value ";
     }
     stopTest();
     return ( testcases );
