@@ -34,16 +34,17 @@
 #include "ChromeClientQt.h"
 #include "Font.h"
 #include "Image.h"
+#include "InitializeLogging.h"
+#include "MemoryCache.h"
 #include "NotImplemented.h"
 #include "Page.h"
 #include "PlatformStrategiesQt.h"
 #include "RenderThemeQStyle.h"
+#include "RuntimeEnabledFeatures.h"
 #include "ScriptController.h"
 #include "ScrollbarThemeQStyle.h"
 #include "SecurityPolicy.h"
-#if USE(QTKIT)
-#include "WebSystemInterface.h"
-#endif
+#include "Settings.h"
 
 #include "qwebelement_p.h"
 #include <JavaScriptCore/runtime/InitializeThreading.h>
@@ -62,7 +63,7 @@ static WebCore::QStyleFacade* createStyleForPage(WebCore::Page* page)
 {
     QWebPageAdapter* pageAdapter = 0;
     if (page)
-        pageAdapter = static_cast<WebCore::ChromeClientQt*>(page->chrome()->client())->m_webPage;
+        pageAdapter = static_cast<WebCore::ChromeClientQt*>(page->chrome().client())->m_webPage;
     return initCallback(pageAdapter);
 }
 
@@ -72,11 +73,7 @@ Q_DECL_EXPORT void initializeWebKitQt()
     if (initCallback) {
         WebCore::RenderThemeQStyle::setStyleFactoryFunction(createStyleForPage);
         WebCore::RenderThemeQt::setCustomTheme(WebCore::RenderThemeQStyle::create, new WebCore::ScrollbarThemeQStyle);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
-        // Only enable kerning by default in Qt 5.1 where it can use the fast font path.
-        // In Qt 5.0 this would have forced the complex font path.
         WebCore::Font::setDefaultTypesettingFeatures(WebCore::Kerning);
-#endif
     }
 }
 
@@ -105,9 +102,10 @@ Q_DECL_EXPORT void initializeWebCoreQt()
     PlatformStrategiesQt::initialize();
     QtWebElementRuntime::initialize();
 
-#if USE(QTKIT)
-    InitWebCoreSystemInterface();
-#endif
+    if (!WebCore::memoryCache()->disabled())
+        WebCore::memoryCache()->setDeadDecodedDataDeletionInterval(60);
+    WebCore::RuntimeEnabledFeatures::setCSSCompositingEnabled(true);
+    WebCore::Settings::setDefaultMinDOMTimerInterval(0.004);
 
     initialized = true;
 }
